local addonName, addonData = ...;

DLU = {
	prefix = "[DLU]",
	addonColor = GetAddOnMetadata(addonName, "X-AddonColor")
};
DLU.Settings = {};
DLU.Settings.Frames = {};
addonData.Settings = {};

-- DLUHQ
DLUHQ = CreateFrame("Frame", "DLUHQ_UI", UIParent, BackdropTemplateMixin and "BackdropTemplate");
DLUHQ.Panels = {};
DLUHQ.Container = CreateFrame("Frame", nil, DLUHQ);

-- Frames
DLU.CoreFrame = CreateFrame("Frame");
DLU.EventFrame = CreateFrame("Frame");

-- Variables
local helpList = nil;

-- Settings
function DLU.getDefaultOptions()
	return {
		showDLUHQOnLogon = false,
		showLoadMessage = true,
		repFormat = "Current / Total",
		seenNotice = false,
		enableSellGreyButton = true,
		hideDefaultBars = false,
	};
end

function DLU.getDefaultCharacterSettings()
	return {
		autoRepair = false,
		questingMode = false,
		FavoriteMounts = {
			ground = nil,
			flying = nil,
			dragonRiding = nil
		},
		skipCinematics = false
	};
end

function DLU.getSettings()
	DLUSettings = DLUSettings or DLU.getDefaultOptions();
	DLUCharacterSettings = DLUCharacterSettings or DLU.getDefaultCharacterSettings();
end

function DLU.ResetCharacterSettings()
	DLUCharacterSettings = DLU.getDefaultCharacterSettings();
end

function DLU.ResetGlobalSettings()
	DLUSettings = DLU.getDefaultOptions();
end

function DLU.resetAddonOptions()
	DLU.ResetGlobalSettings();
	DLU.ResetCharacterSettings();
end

function DLU.UseNewUI()
	return false;
end

-- Methods
function DLU.FormatIntegers(number)
	local i, j, minus, int, fraction = tostring(number):find('([-]?)(%d+)([.]?%d*)')

	-- reverse the int-string and append a comma to all blocks of 3 digits
	int = int:reverse():gsub("(%d%d%d)", "%1,")

	-- reverse the int-string back remove an optional comma and put the 
	-- optional minus and fractional part back
	return minus .. int:reverse():gsub("^,", "") .. fraction
end

function DLU.colorizeString(text, colorType)
	if (colorType == "red") then
		return "|cffCC0000" .. text .. "|r";
	elseif (colorType == "green") then
		return "|cff00CC00" .. text .. "|r";
	elseif (colorType == "blue") then
		return "|cff0000CC" .. text .. "|r";
	elseif (colorType == "orange") then
		return "|cffffa500" .. text .. "|r";
	elseif (colorType == "black") then
		return "|cff000000" .. text .. "|r";
	elseif (colorType == "purple") then
		return "|cff8510d8" .. text .. "|r";
	elseif (colorType == "darkgray") then
		return "|cff444444" .. text .. "|r";
	elseif (colorType == "expansion") then
		return format("|c%s%s|r", DLU.addonColor, text);
	else
		return text;
	end
end

function DLU.XpLeft()
	local xpToNextLevel = (UnitXPMax("player") - UnitXP("player"));
	local xpRested = GetXPExhaustion();
	local xpRestedPercentage;
	local xpRestedFormat;
	
	if (xpRested ~= nil) then
		xpRestedPercentage = (xpRested / UnitXPMax("player")) * 100;
	end
	
	local xpPercentageGained = floor((UnitXP("player") / UnitXPMax("player")) * 100);
	local xpPercentageNeeded = 100 - xpPercentageGained;

	local maxLevel = GetMaxLevelForPlayerExpansion();
	
	local xpFormat = DLU.FormatIntegers(xpToNextLevel);
	
	if (xpRested) then
		xpRestedFormat = DLU.FormatIntegers(xpRested);
	end
	
	return xpFormat, xpRestedFormat, xpPercentageNeeded, xpRestedPercentage, maxLevel;
end

function DLU.createMessage(message, messageType)
	local prefixText = DLU.colorizeString(DLU.prefix, "expansion");
	local text = format("%s %s", prefixText, message);

	if (messageType == "error") then
		text = format("%s %s", prefixText, DLU.colorizeString(message, "red"));
	end

	print(text);
end

function DLU.itemColorString(num, text)
	if (text ~= nil) then
		local color = ITEM_QUALITY_COLORS[num];
		local result = color.hex .. text .. "|r";
		return result;
	end
	
	return;
end

function DLU.GetFormattedTime(number)
	local d, h, m, s = ChatFrame_TimeBreakDown(number);
	return format("%id, %ih, %im %is", d,h,m,s);
end

function DLU.GetClassColorCode()
	local _, className = UnitClass("player");
	local color = RAID_CLASS_COLORS[className];
    local r,g,b = Round(color.r * 255), Round(color.g * 255), Round(color.b * 255);
    return format("%02X%02X%02X", r, g, b);
end

-- Other methods
function addonData:CheckSettings()
	local settings = DLU.getDefaultOptions();
	for k,v in pairs(settings) do
		if (DLUSettings[k] == nil) then
			DLUSettings[k] = v;
		end
	end
end

local function CreateHelpList()
	helpList = {
		[1] = {
			["command"] = SLASH_RELOADUI1,
			["text"] = addonData.L.Reload
		},
		[2] = {
			["command"] = SLASH_DLU1 .. " hq",
			["text"] = addonData.L.DLUHQ
		},
		[3] = {
			["command"] = SLASH_DLU1 .. " mountup",
			["text"] = addonData.L.MountUpInfo
		}
	};
end

local function Help()
	print(format(addonData.L.HelpMsg, addonName));
	if (helpList == nil) then
		CreateHelpList();
	end
	
	for k,v in ipairs(helpList) do
		local command = DLU.colorizeString(v["command"], "darkgray");
		local text = v["text"];
		print(format("%s: %s", command, text));
	end
end

local function Test()
	local currentDate = C_DateAndTime.GetCurrentCalendarTime();
	local day = currentDate.monthDay;
    local month = 0;
	local todayEventsCount = C_Calendar.GetNumDayEvents(month, day);
	for i=1, todayEventsCount do
		local todayEvent = C_Calendar.GetDayEvent(month, day, i);
		if (todayEvent ~= nil) then
			if (string.find(todayEvent.title, "WTS")) then
				print(string.format("Found potential ad (%s)", todayEvent.title));
			else
				print(todayEvent.title, todayEvent.eventType, todayEvent.inviteStatus);
			end
		end
	end
end

function DLU:SlashCmdHandler(msg)
	local command, arg2, arg3 = strsplit(" ", msg);
	local arg = command:lower();
	if (arg2 ~= nil) then
		arg2 = arg2:lower();
	end
	if (arg3 ~= nil) then
		arg3 = arg3:lower();
	end
	
	if (arg == "help") then
		Help();
	elseif (arg == "setsettings") then
		LoadCharacterSettings();
	elseif (arg == "xpleft") then
		print(DLU.XpLeft());
	elseif (arg == "hq") then
		DLUHQ:Show();
	elseif (arg == "mountup") then
		DLU.MountUp();
	elseif (arg == "test") then
		Test();
	end
end

-- Addon commands --
SLASH_RELOADUI1 = "/rl";
SlashCmdList.RELOADUI = ReloadUI;

SLASH_DLU1 = "/dlu";
SlashCmdList.DLU = function(commandName)
	DLU:SlashCmdHandler(commandName);
end

-- Registering Event --
DLU.CoreFrame:RegisterEvent("PLAYER_LOGIN");
DLU.CoreFrame:SetScript("OnEvent", function()
	DLU.getSettings();
	
	addonData:CheckSettings();
	
	if (DLUSettings.showLoadMessage) then
		DLU.createMessage(format(addonData.L.LoadedMsg, addonName));
	end

	if (DLU.UseNewUI() and not DLUSettings.seenNotice) then
		DLU.createMessage(addonData.L.AddonNotice);
		DLUSettings.seenNotice = true;
	end

	DLU.CoreFrame:UnregisterEvent("PLAYER_LOGIN")
end);